from typing import Dict, List

from magic_eden.api.base import MagicEdenOfficialApi
from magic_eden.api.utils.consts import MEAPITokensUrlBuilder
from magic_eden.api.utils.data import list_map
from magic_eden.api.utils.data import token_response_mapper, listing_response_mapper, \
    offer_response_mapper, activity_response_mapper
from magic_eden.api.utils.types import METoken, MEListingItem, MEOffer, MEActivity


class MagicEdenTokensApi(MagicEdenOfficialApi):
    url_builder_class = MEAPITokensUrlBuilder
    url_builder: MEAPITokensUrlBuilder

    def get_token_dirty(
            self,
            token: str,
            request_kwargs: dict = None
    ) -> Dict:
        url = self.url_builder.get_token(token)
        return self._get_request(url, request_kwargs=request_kwargs)

    def get_token(
            self,
            token: str,
            request_kwargs: dict = None
    ) -> METoken:
        dirty = self.get_token_dirty(token, request_kwargs=request_kwargs)
        if dirty:
            return token_response_mapper(dirty)

    def listings_dirty(
            self,
            token: str,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.listings(token)
        return self._get_request(url, request_kwargs=request_kwargs)

    def listings(
            self,
            token: str,
            request_kwargs: dict = None
    ) -> List[MEListingItem]:
        dirty = self.listings_dirty(token, request_kwargs=request_kwargs)
        return list_map(listing_response_mapper, dirty)

    def offer_received_dirty(
            self,
            token: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.offer_received(token, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def offer_received(
            self,
            token: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEOffer]:
        dirty = self.offer_received_dirty(token, offset, limit, request_kwargs=request_kwargs)
        return list_map(offer_response_mapper, dirty)

    def activities_dirty(
            self,
            token: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.activities(token, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def activities(
            self,
            token: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEActivity]:
        dirty = self.activities_dirty(token, offset, limit, request_kwargs=request_kwargs)
        return list_map(activity_response_mapper, dirty)



from typing import Type

from .utils.api import BaseApi
from .utils.consts import MEUrlBuilder
from .utils.types import MagicEdenAPIEnvironment


class MagicEdenOfficialApi(BaseApi):
    url_builder_class: Type[MEUrlBuilder]
    url_builder: MEUrlBuilder

    def __init__(self, environment: MagicEdenAPIEnvironment = 'MAINNET', request_kwargs: dict = None):
        default_headers = {'ME-Pub-API-Metadata': '{"paging":true}'}
        super(MagicEdenOfficialApi, self).__init__(request_kwargs={
            "headers": default_headers,
            **(request_kwargs or {})
        })
        self.url_builder = self.url_builder_class(environment)



from typing import List, Dict

from magic_eden.api.base import MagicEdenOfficialApi
from magic_eden.api.utils.consts import MEAPIWalletUrlsBuilder
from magic_eden.api.utils.data import list_map
from magic_eden.api.utils.data import token_response_mapper, activity_response_mapper, \
    offer_response_mapper, \
    escrow_balance_response_mapper
from magic_eden.api.utils.types import MEActivity, METoken, MEOffer, MEEscrowBalance


class MagicEdenWalletsApi(MagicEdenOfficialApi):
    url_builder_class = MEAPIWalletUrlsBuilder
    url_builder: MEAPIWalletUrlsBuilder

    def tokens_dirty(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            listed_only: bool = False,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.tokens(wallet_address, offset, limit, listed_only)
        return self._get_request(url, request_kwargs=request_kwargs)

    def tokens(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            listed_only: bool = False,
            request_kwargs: dict = None
    ) -> List[METoken]:
        dirty = self.tokens_dirty(wallet_address, offset, limit, listed_only, request_kwargs=request_kwargs)
        return list_map(token_response_mapper, dirty)

    def activities_dirty(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.activities(wallet_address, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def activities(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEActivity]:
        dirty = self.activities_dirty(wallet_address, offset, limit, request_kwargs=request_kwargs)
        return list_map(activity_response_mapper, dirty)

    def offers_made_dirty(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.offers_made(wallet_address, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def offers_made(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEOffer]:
        dirty = self.offers_made_dirty(wallet_address, offset, limit, request_kwargs=request_kwargs)
        return list_map(offer_response_mapper, dirty)

    def offers_received_dirty(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.offers_received(wallet_address, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def offers_received(
            self,
            wallet_address: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEOffer]:
        dirty = self.offers_received_dirty(wallet_address, offset, limit, request_kwargs=request_kwargs)
        return list_map(offer_response_mapper, dirty)

    def escrow_balance_dirty(
            self,
            wallet_address: str,
            request_kwargs: dict = None
    ) -> Dict:
        url = self.url_builder.escrow_balance(wallet_address)
        return self._get_request(url, request_kwargs=request_kwargs)

    def escrow_balance(
            self,
            wallet_address: str,
            request_kwargs: dict = None
    ) -> MEEscrowBalance:
        dirty = self.escrow_balance_dirty(wallet_address, request_kwargs=request_kwargs)
        if dirty:
            return escrow_balance_response_mapper(dirty)



from typing import List, Dict

from magic_eden.api.base import MagicEdenOfficialApi
from magic_eden.api.utils.consts import MEAPICollectionsUrlsBuilder
from magic_eden.api.utils.data import list_map
from magic_eden.api.utils.data import listing_response_mapper, activity_response_mapper, \
    collection_stats_mapper
from magic_eden.api.utils.types import MECollection, MEActivity, MECollectionStats, MEListingItem


class MagicEdenCollectionsApi(MagicEdenOfficialApi):
    url_builder_class = MEAPICollectionsUrlsBuilder
    url_builder: MEAPICollectionsUrlsBuilder

    def collections_dirty(
            self,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.collections(offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def collections(
            self,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MECollection]:
        return self.collections_dirty(offset, limit, request_kwargs=request_kwargs)

    def listings_dirty(
            self,
            symbol: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.listings(symbol, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def listings(
            self,
            symbol: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEListingItem]:
        dirty = self.listings_dirty(symbol, offset, limit, request_kwargs=request_kwargs)
        return list_map(listing_response_mapper, dirty)

    def activities_dirty(
            self,
            symbol: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.activities(symbol, offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def activities(
            self,
            symbol: str,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MEActivity]:
        dirty = self.activities_dirty(symbol, offset, limit, request_kwargs=request_kwargs)
        return list_map(activity_response_mapper, dirty)

    def stats_dirty(
            self,
            symbol: str,
            request_kwargs: dict = None
    ) -> Dict:
        url = self.url_builder.stats(symbol)
        return self._get_request(url, request_kwargs=request_kwargs)

    def stats(
            self,
            symbol,
            request_kwargs: dict = None
    ) -> MECollectionStats:
        dirty = self.stats_dirty(symbol, request_kwargs=request_kwargs)
        return None if dirty is None else collection_stats_mapper(dirty)
